// e01_normalize.cpp:normalize histograms to plot
#include "opencv\cv.hpp"
#include <iostream>

using namespace cv;
using namespace std;

int main(int argc, char* argv[])
{
	// Objects
	Mat original_img;

	// Load image from disk
	original_img = imread("rayosx.tif", CV_LOAD_IMAGE_GRAYSCALE);
	if (!original_img.data){
		cout << "error loading image" << endl;
		return 1;
	}
	// Initialize histogram parameters
	/// Establish the number of bins
	int histSize = 256;
	/// Set the ranges GrayScale 0-255
	float range[] = { 0, 256 };
	const float* histRange = { range };
	/// Set histogram image
	int hist_w = 512; int hist_h = 400;
	int bin_w = cvRound((double)hist_w / histSize);
	Mat histImage(hist_h, hist_w, CV_8UC3, Scalar(0, 0, 0));

	// Calculate histogram
	Mat original_hist, normalized_hist;
	calcHist(&original_img, 1, 0, Mat(), original_hist, 1, &histSize, &histRange, true, false);
	
	// Print the values of the original histogram on console
	cout << "Original histogram" << endl;
	for (int h = 0; h < histSize; h++)
	{
		float binVal = original_hist.at<float>(h);
		cout << " " << binVal;
	}
	cout << endl;

	// Min max localization
	Point min_point, max_point;
	double min, max;
	minMaxLoc(original_hist, &min, &max, &min_point, &max_point, Mat());

	// Print min and max value and its localization
	cout << "minimun value: " << min << " at index:" << min_point.y << endl;
	cout << "Maximun value: " << max << " at index:" << max_point.y << endl << endl;

	/// Normalize the result to [ 0, histImage.rows ]
	normalize(original_hist, normalized_hist, 0, histImage.rows, NORM_MINMAX, -1, Mat());

	// Print the values on console
	cout << "Normalized histogram" << endl;
	for (int h = 0; h < histSize; h++)
	{
		float binVal = normalized_hist.at<float>(h);
		cout << " " << binVal;
	}
	cout << endl;

	minMaxLoc(normalized_hist, &min, &max, &min_point, &max_point, Mat());

	// Print min and max value and its localization
	cout << "minimun value: " << min << " at index:" << min_point.y << endl;
	cout << "Maximun value: " << max << " at index:" << max_point.y << endl;

	/// Plot histogram
	for (int i = 1; i < histSize; i++)
	{
		// Line from the previous vale to the current one. Unfilled. Width 2 (bin_w)
		//line(histImage, 
		//	Point(bin_w*(i - 1), hist_h - cvRound(normalized_hist.at<float>(i - 1))),
		//	Point(bin_w*(i), hist_h - cvRound(normalized_hist.at<float>(i))),
		//	Scalar(0, 255, 0), 2, 8, 0);
		
		//	Line of width 2 (bin_w = 512 width / 256 gray scale values) filled
		line(histImage, 
			Point(bin_w*(i), hist_w),
			Point(bin_w*(i), hist_h - cvRound(normalized_hist.at<float>(i))),
			Scalar(255, 0, 0), bin_w, 8, 0);
	}

	// Windows for all the images
	namedWindow("Original picture", CV_WINDOW_AUTOSIZE);
	namedWindow("histogram", CV_WINDOW_AUTOSIZE);

	// Show image in the name of the window
	imshow("Original picture", original_img);
	//imshow("Normalized picture", normalized_img);
	imshow("histogram", histImage);

	// Function for show the image in ms.
	waitKey(0);
	// Free memory
	histImage.release();
	original_hist.release();
	destroyAllWindows();
	// End of the program
	return 0;
}

